// PDFViewController.m

#import "PDFViewController.h"
#import "PDFView.h"
#import "PDFScrollView.h"

@interface PDFViewController (private)

// 表示の更新
- (void)_renewPages;

@end

@implementation PDFViewController

//--------------------------------------------------------------//
#pragma mark -- View --
//--------------------------------------------------------------//

- (void)viewDidLoad
{
    _subScrollView.controller = self;
    
    // PDFドキュメントの作成
    NSString*   path;
    NSURL*      url;
    path = [[NSBundle mainBundle] pathForResource:@"sample" ofType:@"pdf"];
    url = [NSURL fileURLWithPath:path];
    _document = CGPDFDocumentCreateWithURL((CFURLRef)url);
    
    // innerViewをメインスクロールビューに追加
    [_mainScrollView addSubview:_innerView];
    
    // メインスクロールビューのコンテントサイズを設定
    _mainScrollView.contentSize = _innerView.frame.size;
    
    // インデックスの初期値として-1を設定
    _index = -1;
}

- (void)viewWillAppear:(BOOL)animated
{
    // 親クラスの呼び出し
    [super viewWillAppear:animated];
    
    // ページの更新
    [self _renewPages];
}

//--------------------------------------------------------------//
#pragma mark -- Image --
//--------------------------------------------------------------//

- (PDFView*)_createPdfViewWithIndex:(int)index
{
    // PDF viewを作成
    PDFView*    pdfView;
    pdfView = [[PDFView alloc] initWithFrame:CGRectZero];
    [pdfView autorelease];
    
    // PDFページを取得
    CGPDFPageRef    page = NULL;
    if (index > 0 || index <= CGPDFDocumentGetNumberOfPages(_document)) {
        page = CGPDFDocumentGetPage(_document, index);
    }
    pdfView.page = page;
    
    // PDFの大きさを取得
    CGRect  pageRect = CGRectZero;
    float   scale = 1.0f;
    if (page) {
        pageRect = CGPDFPageGetBoxRect(page, kCGPDFMediaBox);
    }
    if (CGRectGetWidth(pageRect) > 0) {
        scale = CGRectGetWidth(self.view.frame) / CGRectGetWidth(pageRect);
    }
    
    // 初期のPDF表示の大きさおよびスケールを設定
    pageRect.size.width *= scale;
    pageRect.size.height *= scale;
    pdfView.frame = pageRect;
    pdfView.scale = scale;
    
    return pdfView;
}

- (void)_renewPages
{
    CGRect          rect;
    
    // 現在のインデックスを保存
    int oldIndex = _index;
    
    // PDFのページ数を取得
    int pageNumber;
    pageNumber = CGPDFDocumentGetNumberOfPages(_document);
    
    // コンテントオフセットを取得
    CGPoint offset;
    offset = _mainScrollView.contentOffset;
    if (offset.x == 0) {
        // 前のページへ移動
        _index--;
    }
    if (offset.x >= _mainScrollView.contentSize.width - CGRectGetWidth(_mainScrollView.frame)) {
        // 次のページへ移動
        _index++;
    }
    
    // インデックスの値をチェック
    if (_index < 1) {
        _index = 1;
    }
    if (_index > pageNumber) {
        _index = pageNumber;
    }
    
    if (_index == oldIndex) {
        return;
    }
    
    //
    // 左側のPDF viewを更新
    //
    
    // 古いPDF Viewを解放
    [_pdfView0 removeFromSuperview];
    [_pdfView0 release], _pdfView0 = nil;
    
    // PDF Viewを作成
    _pdfView0 = [[self _createPdfViewWithIndex:_index - 1] retain];
    
    // 表示位置の設定
    rect.size = _pdfView0.frame.size;
    rect.origin = CGPointZero;
    if (!CGSizeEqualToSize(_pdfView0.frame.size, CGSizeZero)) {
        rect.origin.x += (CGRectGetWidth(self.view.frame) - CGRectGetWidth(rect)) * 0.5f;
        rect.origin.y += (CGRectGetHeight(self.view.frame) - CGRectGetHeight(rect)) * 0.5f;
    }
    _pdfView0.frame = rect;
    [_mainScrollView addSubview:_pdfView0];
    
    //
    // 中央のPDF view、サブスクロールビューを更新
    //
    
    // サブスクロールビューのframe
    rect.origin.x = CGRectGetMaxX(_pdfView0.frame) > 0 ?
            CGRectGetMaxX(_pdfView0.frame) + 20.0f : 0;
    rect.origin.y = 0;
    rect.size = self.view.frame.size;
    
    // サブスクロールビューの設定
    _subScrollView.frame = rect;
    
    // 古いPDF Viewを解放
    [_pdfView1 removeFromSuperview];
    [_pdfView1 release], _pdfView1 = nil;
    
    // PDF Viewを作成
    _pdfView1 = [[self _createPdfViewWithIndex:_index] retain];
    
    // 表示位置の設定
    rect.size = _pdfView1.frame.size;
    rect.origin = CGPointZero;
    if (!CGSizeEqualToSize(_pdfView1.frame.size, CGSizeZero)) {
        rect.origin.x += (CGRectGetWidth(self.view.frame) - CGRectGetWidth(rect)) * 0.5f;
        rect.origin.y += (CGRectGetHeight(self.view.frame) - CGRectGetHeight(rect)) * 0.5f;
    }
    _pdfView1.frame = rect;
    [_subScrollView addSubview:_pdfView1];
    
    // サブスクロールビューのコンテントサイズを設定
    _subScrollView.contentSize = rect.size;
    
    //
    // 右側のPDF viewを更新
    //
    
    // 古いPDF Viewを解放
    [_pdfView2 removeFromSuperview];
    [_pdfView2 release], _pdfView2 = nil;
    
    // PDF Viewを作成
    _pdfView2 = [[self _createPdfViewWithIndex:_index + 1] retain];
    
    // 表示位置の設定
    rect.size = _pdfView2.frame.size;
    rect.origin.x = CGRectGetMaxX(_subScrollView.frame) + 20.0f;
    rect.origin.y = 0;
    if (!CGSizeEqualToSize(_pdfView2.frame.size, CGSizeZero)) {
        rect.origin.x += (CGRectGetWidth(self.view.frame) - CGRectGetWidth(rect)) * 0.5f;
        rect.origin.y += (CGRectGetHeight(self.view.frame) - CGRectGetHeight(rect)) * 0.5f;
    }
    _pdfView2.frame = rect;
    [_mainScrollView addSubview:_pdfView2];
    
    //
    // メインスクロールビューの更新
    //
    
    // コンテントサイズとオフセットの設定
    CGSize  size;
    size.width = _index > 1 && _index < pageNumber ? 
            (CGRectGetWidth(self.view.frame) + 20.0f) * 3.0f : 
            (CGRectGetWidth(self.view.frame) + 20.0f) * 2.0f;
    size.height = 0;
    _mainScrollView.contentSize = size;
    _mainScrollView.contentOffset = _subScrollView.frame.origin;
}

- (void)frameToCenter
{
    // 現在のビューのサイズを取得
    CGSize size;
    size = self.view.bounds.size;
    
    // PDFビューのframeを取得
    CGRect  pdfFrame;
    pdfFrame = _pdfView1.frame;
    
    // 横方向の中央に移動
    if (CGRectGetWidth(pdfFrame) < size.width) {
        pdfFrame.origin.x = (size.width - CGRectGetWidth(pdfFrame)) * 0.5f;
    }
    else {
        pdfFrame.origin.x = 0;
    }
    
    // 縦方向の中央に移動
    if (CGRectGetHeight(pdfFrame) < size.height) {
        pdfFrame.origin.y = (size.height - CGRectGetHeight(pdfFrame)) * 0.5f;
    }
    else {
        pdfFrame.origin.y = 0;
    }
    
    // PDFビューのframeを設定
    _pdfView1.frame = pdfFrame;
}

//--------------------------------------------------------------//
#pragma mark -- UIScrollViewDelegate --
//--------------------------------------------------------------//

- (void)scrollViewDidEndDragging:(UIScrollView*)scrollView 
        willDecelerate:(BOOL)decelerate
{
    // メインスクロールビューの場合
    if (scrollView == _mainScrollView) {
        if (!decelerate) {
            // ページの更新
            [self _renewPages];
        }
    }
}

- (void)scrollViewDidEndDecelerating:(UIScrollView*)scrollView
{
    // メインスクロールビューの場合
    if (scrollView == _mainScrollView) {
        // ページの更新
        [self _renewPages];
    }
}

- (UIView*)viewForZoomingInScrollView:(UIScrollView*)scrollView
{
    // サブスクロールビューの場合
    if (scrollView == _subScrollView) {
        // 中央のPDF viewを使う
        return _pdfView1;
    }
    
    return nil;
}

@end
